@@ -23,7 +23,9 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
                          conf_int=FALSE, conf_level=0.95, times=NULL,
                          bootstrap=FALSE, n_boot=500, n_cores=1,
                          na.action=options()$na.action,
-                         clean_data=TRUE, ...) {
+                         clean_data=TRUE, iso_reg=FALSE,
+                         force_bounds=FALSE, mi_extrapolation=FALSE,
+                         ...) {
 
   # use data.frame methods only, no tibbles etc.
   if (inherits(data, "data.frame")) {
@@ -48,16 +50,6 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
   ## using multiple imputation
   if (inherits(data, "mids")) {
 
-    # get event specific times
-    if (is.null(times)) {
-      times <- sort(unique(data$data[, ev_time][data$data[, event]==1]))
-
-      # add zero if not already in there
-      if (!0 %in% times) {
-        times <- c(0, times)
-      }
-    }
-
     # levels of the group variable
     if (is.numeric(data$data[, variable])) {
       levs <- unique(data$data[, variable])
@@ -68,6 +60,16 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
     # transform to long format
     mids <- mice::complete(data, action="long", include=FALSE)
 
+    # get event specific times (pooled over all imputed datasets)
+    if (is.null(times)) {
+      times <- sort(unique(mids[, ev_time][mids[, event]==1]))
+
+      # add zero if not already in there
+      if (!0 %in% times & method!="tmle") {
+        times <- c(0, times)
+      }
+    }
+
     # get additional arguments
     args <- list(...)
 
@@ -144,8 +146,8 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
       # use Rubins Rule
       plotdata <- dats %>%
         dplyr::group_by(., time, group) %>%
-        dplyr::summarise(surv=mean(surv),
-                         se=mean(se),
+        dplyr::summarise(surv=mean(surv, na.rm=mi_extrapolation),
+                         se=mean(se, na.rm=mi_extrapolation),
                          .groups="drop_last")
       plotdata <- as.data.frame(plotdata)
 
@@ -159,11 +161,32 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
     } else {
       plotdata <- dats %>%
         dplyr::group_by(., time, group) %>%
-        dplyr::summarise(surv=mean(surv),
+        dplyr::summarise(surv=mean(surv, na.rm=mi_extrapolation),
                          .groups="drop_last")
       plotdata <- as.data.frame(plotdata)
     }
 
+    # if estimated survival times beyond the maximum observed survival time
+    # exist, remove them if specified
+    if (!mi_extrapolation) {
+      max_t_group <- max_observed_time(mids=data, variable=variable,
+                                       ev_time=ev_time, event=event,
+                                       cause=1, levs=levs, method=method,
+                                       type="surv")
+      plotdata <- merge(plotdata, max_t_group, by="group", all.x=TRUE)
+      plotdata <- plotdata[plotdata$time <= plotdata$max_t,]
+      plotdata$max_t <- NULL
+      plotdata <- plotdata[order(plotdata$group, plotdata$time), ]
+    }
+
+    if (force_bounds) {
+      plotdata <- force_bounds_surv(plotdata)
+    }
+
+    if (iso_reg) {
+      plotdata <- iso_reg_surv(plotdata)
+    }
+
     # output object
     out_obj <- list(mids_analyses=out,
                     adjsurv=plotdata,
@@ -176,8 +199,8 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
 
       plotdata_boot <- boot_dats %>%
         dplyr::group_by(., time, group) %>%
-        dplyr::summarise(surv=mean(surv),
-                         se=mean(se),
+        dplyr::summarise(surv=mean(surv, na.rm=mi_extrapolation),
+                         se=mean(se, na.rm=mi_extrapolation),
                          .groups="drop_last")
       plotdata_boot <- as.data.frame(plotdata_boot)
 
@@ -240,7 +263,7 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
       times <- sort(unique(data[, ev_time][data[, event]==1]))
 
       # add zero if not already in there
-      if (!0 %in% times) {
+      if (!0 %in% times & method!="tmle") {
         times <- c(0, times)
       }
     }
@@ -295,7 +318,8 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
                                              method=method,
                                              times=times, i=i,
                                              surv_fun=surv_fun,
-                                             na.action=na.action, ...)
+                                             na.action=na.action,
+                                             ...)
         }
       }
 
@@ -331,6 +355,14 @@ adjustedsurv <- function(data, variable, ev_time, event, method,
     # keep factor levels in same order as data
     plotdata$group <- factor(plotdata$group, levels=levs)
 
+    if (force_bounds) {
+      plotdata <- force_bounds_surv(plotdata)
+    }
+
+    if (iso_reg) {
+      plotdata <- iso_reg_surv(plotdata)
+    }
+
     out <- list(adjsurv=plotdata,
                 data=data,
                 method=method,
@@ -455,6 +487,14 @@ summary.adjustedsurv <- function(object, ...) {
     method_name <- "Stratification & Weighting by Amato"
   } else if (object$method=="strat_nieto") {
     method_name <- "Stratification & Weighting by Gregory / Nieto & Coresh"
+  } else if (object$method=="tmle") {
+    method_name <- "Targeted Maximum Likelihood Estimator"
+  } else if (object$method=="iv_2SRIF") {
+    method_name <- "Instrumental Variable Estimator (2SRI-F)"
+  } else if (object$method=="prox_iptw") {
+    method_name <- "Proximal Causal Inference Based IPTW"
+  } else if (object$method=="prox_aiptw") {
+    method_name <- "Proximal Causal Inference Based AIPTW"
   }
 
   times_str <- ifelse(is.null(object$call$times), "Event-Specific Times",
